# -*- coding: utf-8 -*-

__title__ = 'drf-firebase3-auth'
__version__ = '0.0.5'
__author__ = 'Baryshnikov Nikolay'
__license__ = 'MIT'
__copyright__ = 'Copyright 2019 Baryshnikov Nikolay'

# Version synonym
VERSION = __version__
